@@ -1,5 +1,5 @@
 #! perl -w
-# $Revision: 1.103 $
+# $Revision: 1.104.2.1 $
 
 # Copyright (c) 2000-2002 Ned Konz. All rights reserved.  This program is free
 # software; you can redistribute it and/or modify it under the same terms as
@@ -41,7 +41,7 @@ BEGIN
 {
 	require Exporter;
 
-	$VERSION = "1.12_03";
+	$VERSION = "1.15_01";
 	@ISA = qw( Exporter );
 
 	my @ConstantNames = qw( FA_MSDOS FA_UNIX GPBF_ENCRYPTED_MASK
@@ -637,6 +637,7 @@ sub extractMember    # Archive::Zip::Archive
 	my $self   = shift;
 	my $member = shift;
 	$member = $self->memberNamed($member) unless ref($member);
+    my $originalSize = $member->compressedSize();
 	return _error('member not found') unless $member;
 	my $name = shift;    # local FS name if given
 	my ( $volumeName, $dirName, $fileName );
@@ -657,7 +658,10 @@ sub extractMember    # Archive::Zip::Archive
 		mkpath($dirName);
 		return _ioError("can't create dir $dirName") if ( !-d $dirName );
 	}
-	return $member->extractToFileNamed( $name, @_ );
+	my $rc = $member->extractToFileNamed( $name, @_ );
+    # TODO refactor this fix into extractToFileNamed()
+    $member->{'compressedSize'} = $originalSize;
+    return $rc; 
 }
 
 sub extractMemberWithoutPaths    # Archive::Zip::Archive
@@ -665,6 +669,7 @@ sub extractMemberWithoutPaths    # Archive::Zip::Archive
 	my $self   = shift;
 	my $member = shift;
 	$member = $self->memberNamed($member) unless ref($member);
+    my $originalSize = $member->compressedSize(); 
 	return _error('member not found') unless $member;
 	return AZ_OK if $member->isDirectory();
 	my $name = shift;
@@ -674,7 +679,9 @@ sub extractMemberWithoutPaths    # Archive::Zip::Archive
 		$name =~ s{.*/}{};    # strip off directories, if any
 		$name = Archive::Zip::_asLocalName($name);
 	}
-	return $member->extractToFileNamed( $name, @_ );
+	my $rc = $member->extractToFileNamed( $name, @_ );
+    $member->{'compressedSize'} = $originalSize;
+    return $rc;
 }
 
 sub addMember    # Archive::Zip::Archive
@@ -2929,14 +2936,16 @@ sub _readCentralDirectoryFileHeader    # Archive::Zip::ZipFileMember
 		return _ioError("reading central dir header");
 	}
 	my ( $fileNameLength, $extraFieldLength, $fileCommentLength );
-	( $self->{'versionMadeBy'},          $self->{'fileAttributeFormat'},
-	  $self->{'versionNeededToExtract'}, $self->{'bitFlag'},
-	  $self->{'compressionMethod'},      $self->{'lastModFileDateTime'},
-	  $self->{'crc32'},                  $self->{'compressedSize'},
-	  $self->{'uncompressedSize'},       $fileNameLength,
-	  $extraFieldLength,                 $fileCommentLength,
-	  $self->{'diskNumberStart'},        $self->{'internalFileAttributes'},
-	  $self->{'externalFileAttributes'}, $self->{'localHeaderRelativeOffset'} )
+	(
+		$self->{'versionMadeBy'},          $self->{'fileAttributeFormat'},
+		$self->{'versionNeededToExtract'}, $self->{'bitFlag'},
+		$self->{'compressionMethod'},      $self->{'lastModFileDateTime'},
+		$self->{'crc32'},                  $self->{'compressedSize'},
+		$self->{'uncompressedSize'},       $fileNameLength,
+		$extraFieldLength,                 $fileCommentLength,
+		$self->{'diskNumberStart'},        $self->{'internalFileAttributes'},
+		$self->{'externalFileAttributes'}, $self->{'localHeaderRelativeOffset'}
+	)
 	  = unpack( CENTRAL_DIRECTORY_FILE_HEADER_FORMAT, $header );
 
 	$self->{'eocdCrc32'} = $self->{'crc32'};
@@ -2966,6 +2975,13 @@ sub _readCentralDirectoryFileHeader    # Archive::Zip::ZipFileMember
 		}
 	}
 
+	# NK 10/21/04: added to avoid problems with manipulated headers
+	if (    $self->{'uncompressedSize'} != $self->{'compressedSize'}
+		and $self->{'compressionMethod'} == COMPRESSION_STORED )
+	{
+		$self->{'uncompressedSize'} = $self->{'compressedSize'};
+	}
+
 	$self->desiredCompressionMethod( $self->compressionMethod() );
 
 	return AZ_OK;
